package org.jboss.jsr299.tck.spi;


/**
 * Provides Bean related operations.
 * 
 * The TCK porting package must provide an implementation of this interface which is 
 * suitable for the target implementation.
 * 
 * This interface may be removed.
 * 
 * @author Shane Bryzak
 * @author Pete Muir
 * @author David Allen
 * 
 */
public interface Beans
{

   public static final String PROPERTY_NAME = Beans.class.getName();
   
   /**
    * Tests whether the given class is declaring an Enterprise Bean.
    * 
    * @param clazz the type of bean being tested
    * 
    * @return true only if class declares an Enterprise Bean
    */
   public boolean isEnterpriseBean(Class<?> clazz);

   /**
    * Tests whether the given class is declaring a stateful session bean.
    * 
    * @param clazz the type being tested
    * @return true only if class declares a stateful session bean
    */
   public boolean isStatefulBean(Class<?> clazz);

   /**
    * Tests whether the given class is declaring a stateless session bean.
    * 
    * @param clazz the type being tested
    * @return true only if class declares a stateless session bean
    */
   public boolean isStatelessBean(Class<?> clazz);

   /**
    * Tests whether the given class is declaring an entity bean.
    * 
    * @param clazz the type being tested
    * @return true only if class declares an entity bean
    */
   public boolean isEntityBean(Class<?> clazz);
   
   /**
    * Determines if the object instance is actually a proxy object.
    * 
    * @param instance The object which might be a proxy
    * @return true if the object is a proxy
    */
   public boolean isProxy(Object instance);
   
   /**
    * Retrieves the EJB which implements the local interface provided.
    * 
    * @param <T> The local interface type
    * @param beanType The class of the EJB implementing bean
    * @param localInterface The business interface of the bean proxy to return
    * @return the EJB proxy from the container
    */
   public <T> T getEnterpriseBean(Class<? extends T> beanType, Class<T> localInterface);
   
}
